# -----------------------------------------------------------
#  Copyright (C) 2008 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
'''
This module exists solely for version information so I only have to change it
in one place. Based on the suggestion `here. <http://bit.ly/16LbuJF>`_

:author: Dan Blanchard (dan.blanchard@gmail.com)
'''

__version__ = '0.7.8'
VERSION = tuple(int(x) for x in __version__.split('.'))
