#  tests for drmaa-0.7.8-py35_0 (this is a generated file);
print('===== testing package: drmaa-0.7.8-py35_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
try:
    import drmaa
except RuntimeError as e:
    assert str(e) == "Could not find drmaa library.  Please specify its full path using the environment variable DRMAA_LIBRARY_PATH"
#  --- run_test.py (end) ---

print('===== drmaa-0.7.8-py35_0 OK =====');
